@@ -55,6 +55,30 @@ const LsmTraitClass lsm_svg_blending_mode_trait_class = {
 	.to_string = lsm_svg_blending_mode_trait_to_string
 };
 
+static gboolean
+lsm_svg_enable_background_trait_from_string (LsmTrait *abstract_trait, char *string)
+{
+	LsmSvgEnableBackground *trait = (LsmSvgEnableBackground *) abstract_trait;
+
+	*trait = lsm_svg_enable_background_from_string (string);
+
+	return *trait >= 0;
+}
+
+char *
+lsm_svg_enable_background_trait_to_string (LsmTrait *abstract_trait)
+{
+	LsmSvgEnableBackground *trait = (LsmSvgEnableBackground *) abstract_trait;
+
+	return g_strdup (lsm_svg_enable_background_to_string (*trait));
+}
+
+const LsmTraitClass lsm_svg_enable_background_trait_class = {
+	.size = sizeof (LsmSvgEnableBackground),
+	.from_string = lsm_svg_enable_background_trait_from_string,
+	.to_string = lsm_svg_enable_background_trait_to_string
+};
+
 static gboolean
 lsm_svg_length_trait_from_string (LsmTrait *abstract_trait, char *string)
 {
@@ -237,6 +261,23 @@ const LsmTraitClass lsm_svg_matrix_trait_class = {
 	.to_string = lsm_svg_matrix_trait_to_string
 };
 
+static double
+_hue_to_rgb (double m1, double m2, double h)
+{
+	if (h < 0)
+		h = h + 1.0;
+	if (h > 1)
+		h = h - 1.0;
+	if (h * 6.0 < 1.0)
+		return  m1 + (m2 - m1 ) * h * 6.0;
+	if (h * 2.0 < 1.0)
+		return m2;
+       	if (h * 3.0 < 2.0)
+		return m1 + (m2 - m1) * (2.0 / 3.0 - h) * 6.0;
+
+	return m1;
+}
+
 static char *
 _parse_color (char *string,
 	      LsmSvgColor *svg_color,
@@ -313,6 +354,51 @@ _parse_color (char *string,
 			color = 0;
 
 		*paint_type = LSM_SVG_PAINT_TYPE_RGB_COLOR;
+	} else if (strncmp (string, "hsl(", 4) == 0) {
+		double value[3];
+		int i;
+
+		string += 4; /* strlen ("hsl(") */
+
+		lsm_str_skip_spaces (&string);
+
+		for (i = 0; i < 3; i++) {
+			if (!lsm_str_parse_double (&string, &value[i]))
+				break;
+
+			if (i > 0) {
+			       if (*string != '%')
+				break;
+			       string++;
+			}
+
+			lsm_str_skip_comma_and_spaces (&string);
+		}
+
+		lsm_str_skip_spaces (&string);
+
+		if (*string == ')' && i == 3) {
+			double m1, m2;
+			double h,s,l;
+
+			h = value[0] / 360.0;
+			s = value[1] / 100.0; 
+			l = value[2] / 100.0;
+
+			if (l <= 0.5)
+				m2 = l*(s + 1.0);
+			else
+				m2 = l + s - l * s;
+			m1 = l * 2.0 - m2;
+
+			svg_color->red = _hue_to_rgb (m1, m2, h + 1.0/3.0);
+			svg_color->green  = _hue_to_rgb (m1, m2, h);		
+			svg_color->blue = _hue_to_rgb (m1, m2, h - 1.0/3.0);
+
+			*paint_type = LSM_SVG_PAINT_TYPE_RGB_COLOR;
+
+			return string;
+		}
 	} else if (g_strcmp0 (string, "none") != 0) {
 		color = lsm_svg_color_from_string (string);
 
@@ -612,14 +698,21 @@ lsm_svg_dash_array_trait_from_string (LsmTrait *abstract_trait, char *string)
 	dash_array->n_dashes = 0;
 	dash_array->dashes = NULL;
 
-	if (strcmp (string, "none") != 0) {
+	if (strcmp (string, "none") != 0 &&
+	    strcmp (string, "inherit")) {
 		char *iter = (char *) string;
 		unsigned int i;
 
 		while (*iter != '\0') {
-			if (*iter == ',')
+			if (*iter == ',' ||
+			    *iter == ' ') {
 				n_dashes++;
-			iter++;
+				do {
+					iter++;
+				} while (*iter == ',' ||
+					 *iter == ' ');
+			} else
+				iter++;
 		}
 
 		if (n_dashes > 0) {
@@ -651,14 +744,14 @@ lsm_svg_dash_array_trait_from_string (LsmTrait *abstract_trait, char *string)
 				sum += value;
 			}
 		}
-	}
 
-	if (is_error || sum <= 0.0) {
-		g_free (dash_array->dashes);
-		dash_array->n_dashes = 0;
-		dash_array->dashes = NULL;
+		if (is_error || sum <= 0.0) {
+			g_free (dash_array->dashes);
+			dash_array->n_dashes = 0;
+			dash_array->dashes = NULL;
 
-		return !is_error;
+			return !is_error;
+		}
 	}
 
 	return TRUE;
