@@ -33,6 +33,7 @@
 #include <lsmsvgmarkerelement.h>
 #include <lsmsvgclippathelement.h>
 #include <lsmsvgmaskelement.h>
+#include <lsmsvgfiltersurface.h>
 #include <lsmcairo.h>
 #include <lsmstr.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -70,6 +71,12 @@ struct _LsmSvgViewPatternData {
 	double opacity;
 };
 
+typedef struct {
+	cairo_surface_t *surface;
+	double group_opacity;
+	gboolean enable_background;
+} LsmSvgViewBackground;
+
 double
 lsm_svg_view_normalize_length (LsmSvgView *view, const LsmSvgLength *length, LsmSvgLengthDirection direction)
 {
@@ -346,427 +353,6 @@ lsm_svg_view_create_surface_pattern (LsmSvgView *view,
 	return TRUE;
 }
 
-typedef struct {
-	cairo_t *cr;
-	char *ptr;
-	char last_command;
-	double last_cp_x;
-	double last_cp_y;
-	double values[7];
-} LsmSvgPathContext;
-
-/*
- * Code for:
- *
- * 	_calc_angle 
- * 	_cairo_elliptical_arc 
- * 	_cairo_quadratic_curve_to
- * 	_emit_smooth_curve
- * 	_emit_smooth_quadratic_curve
- *
- * is adapted from the goocanvas library (goocanvasutils.c)
- *
- * GooCanvas. Copyright (C) 2005 Damon Chaplin.
- */
-
-static void
-_cairo_quadratic_curve_to (cairo_t *cr, double x1, double y1, double x, double y)
-{
-	double x0, y0, xx1, yy1, xx2, yy2;
-
-	cairo_get_current_point (cr, &x0, &y0);
-
-	/* We need to convert the quadratic into a cubic bezier. */
-
-	xx1 = x0 + (x1 - x0) * 2.0 / 3.0;
-	yy1 = y0 + (y1 - y0) * 2.0 / 3.0;
-
-	xx2 = xx1 + (x - x0) / 3.0;
-	yy2 = yy1 + (y - y0) / 3.0;
-
-	cairo_curve_to (cr, xx1, yy1, xx2, yy2, x, y);
-}
-
-static void
-_cairo_rel_quadratic_curve_to (cairo_t *cr, double dx1, double dy1, double dx, double dy)
-{
-	double x0, y0;
-
-	cairo_get_current_point (cr, &x0, &y0);
-	_cairo_quadratic_curve_to (cr, x0 + dx1, y0 + dy1, x0 + dx, y0 + dy);
-}
-
-static double
-_calc_angle (double ux, double uy, double vx, double vy)
-{
-	double top, u_magnitude, v_magnitude, angle_cos, angle;
-
-	top = ux * vx + uy * vy;
-	u_magnitude = sqrt (ux * ux + uy * uy);
-	v_magnitude = sqrt (vx * vx + vy * vy);
-	angle_cos = top / (u_magnitude * v_magnitude);
-
-	/* We check if the cosine is slightly out-of-bounds. */
-	if (angle_cos >= 1.0)
-		angle = 0.0;
-	if (angle_cos <= -1.0)
-		angle = M_PI;
-	else
-		angle = acos (angle_cos);
-
-	if (ux * vy - uy * vx < 0)
-		angle = - angle;
-
-	return angle;
-}
-
-static void
-_cairo_elliptical_arc (cairo_t *cairo, double rx, double ry, double x_axis_rotation,
-		       gboolean large_arc_flag, gboolean sweep_flag, double x, double y)
-{
-	double x1, y1, x2, y2, lambda;
-	double v1, v2, angle, angle_sin, angle_cos, x11, y11;
-	double rx_squared, ry_squared, x11_squared, y11_squared, top, bottom;
-	double c, cx1, cy1, cx, cy, start_angle, angle_delta;
-
-	cairo_get_current_point (cairo, &x1, &y1);
-
-	x2 = x;
-	y2 = y;
-
-	/* If the endpoints are exactly the same, just return (see SVG spec). */
-	if (x1 == x2 && y1 == y2)
-		return;
-
-	/* If either rx or ry is 0, do a simple lineto (see SVG spec). */
-	if (rx == 0.0 || ry == 0.0)
-	{
-		cairo_line_to (cairo, x2, y2);
-		return;
-	}
-
-	/* Calculate x1' and y1' (as per SVG implementation notes). */
-	v1 = (x1 - x2) / 2.0;
-	v2 = (y1 - y2) / 2.0;
-
-	angle = x_axis_rotation * (M_PI / 180.0);
-	angle_sin = sin (angle);
-	angle_cos = cos (angle);
-
-	x11 = (angle_cos * v1) + (angle_sin * v2);
-	y11 = - (angle_sin * v1) + (angle_cos * v2);
-
-	/* Ensure rx and ry are positive and large enough. */
-	rx = rx > 0.0 ? rx : - rx;
-	ry = ry > 0.0 ? ry : - ry;
-	lambda = (x11 * x11) / (rx * rx) + (y11 * y11) / (ry * ry);
-	if (lambda > 1.0)
-	{
-		gdouble square_root = sqrt (lambda);
-		rx *= square_root;
-		ry *= square_root;
-	}
-
-	/* Calculate cx' and cy'. */
-	rx_squared = rx * rx;
-	ry_squared = ry * ry;
-	x11_squared = x11 * x11;
-	y11_squared = y11 * y11;
-
-	top = (rx_squared * ry_squared) - (rx_squared * y11_squared)
-		- (ry_squared * x11_squared);
-	if (top < 0.0)
-	{
-		c = 0.0;
-	}
-	else
-	{
-		bottom = (rx_squared * y11_squared) + (ry_squared * x11_squared);
-		c = sqrt (top / bottom);
-	}
-
-	if (large_arc_flag == sweep_flag)
-		c = - c;
-
-	cx1 = c * ((rx * y11) / ry);
-	cy1 = c * (- (ry * x11) / rx);
-
-	/* Calculate cx and cy. */
-	cx = (angle_cos * cx1) - (angle_sin * cy1) + (x1 + x2) / 2;
-	cy = (angle_sin * cx1) + (angle_cos * cy1) + (y1 + y2) / 2;
-
-	/* Calculate the start and end angles. */
-	v1 = (x11 - cx1) / rx;
-	v2 = (y11 - cy1) / ry;
-
-	start_angle = _calc_angle (1, 0, v1, v2);
-	angle_delta = _calc_angle (v1, v2, (-x11 - cx1) / rx, (-y11 - cy1) / ry);
-
-	if (sweep_flag == 0 && angle_delta > 0.0)
-		angle_delta -= 2 * M_PI;
-	else if (sweep_flag == 1 && angle_delta < 0.0)
-		angle_delta += 2 * M_PI;
-
-	/* Now draw the arc. */
-	cairo_save (cairo);
-	cairo_translate (cairo, cx, cy);
-	cairo_rotate (cairo, angle);
-	cairo_scale (cairo, rx, ry);
-
-	if (angle_delta > 0.0)
-		cairo_arc (cairo, 0.0, 0.0, 1.0,
-			   start_angle, start_angle + angle_delta);
-	else
-		cairo_arc_negative (cairo, 0.0, 0.0, 1.0,
-				    start_angle, start_angle + angle_delta);
-
-	cairo_restore (cairo);
-}
-
-static void
-_cairo_rel_elliptical_arc (cairo_t *cairo, double rx, double ry, double x_axis_rotation,
-			   gboolean large_arc_flag, gboolean sweep_flag, double dx, double dy)
-{
-	double x, y;
-
-	cairo_get_current_point (cairo, &x, &y);
-
-	_cairo_elliptical_arc (cairo, rx, ry, x_axis_rotation, large_arc_flag, sweep_flag, x +dx ,y + dy);
-}
-
-static void
-_cairo_vertical (cairo_t *cairo, double y)
-{
-	double x0, y0;
-
-	cairo_get_current_point (cairo, &x0, &y0);
-	cairo_line_to (cairo, x0, y);
-}
-
-static void
-_cairo_rel_vertical (cairo_t *cairo, double dy)
-{
-	double x0, y0;
-
-	cairo_get_current_point (cairo, &x0, &y0);
-	cairo_line_to (cairo, x0, y0 + dy);
-}
-
-static void
-_cairo_horizontal (cairo_t *cairo, double x)
-{
-	double x0, y0;
-
-	cairo_get_current_point (cairo, &x0, &y0);
-	cairo_line_to (cairo, x, y0);
-}
-
-static void
-_cairo_rel_horizontal (cairo_t *cairo, double dx)
-{
-	double x0, y0;
-
-	cairo_get_current_point (cairo, &x0, &y0);
-	cairo_line_to (cairo, x0 + dx, y0);
-}
-
-static void
-_emit_function_1 (LsmSvgPathContext *ctxt,
-		  void (*cairo_func) (cairo_t *, double))
-{
-	while (lsm_str_parse_double_list (&ctxt->ptr, 1, ctxt->values) == 1)
-		cairo_func (ctxt->cr, ctxt->values[0]);
-}
-
-static void
-_emit_function_2 (LsmSvgPathContext *ctxt,
-		  void (*cairo_func_a) (cairo_t *, double, double),
-		  void (*cairo_func_b) (cairo_t *, double, double))
-{
-
-	if (lsm_str_parse_double_list (&ctxt->ptr, 2, ctxt->values) == 2) {
-		cairo_func_a (ctxt->cr, ctxt->values[0], ctxt->values[1]);
-
-		/* Why oh why does the specification say Line is implied here ? */
-
-		while (lsm_str_parse_double_list (&ctxt->ptr, 2, ctxt->values) == 2)
-			cairo_func_b (ctxt->cr, ctxt->values[0], ctxt->values[1]);
-	}
-}
-
-static void
-_emit_function_4 (LsmSvgPathContext *ctxt,
-		  void (*cairo_func) (cairo_t *, double, double, double, double))
-{
-	while (lsm_str_parse_double_list (&ctxt->ptr, 4, ctxt->values) == 4)
-		cairo_func (ctxt->cr, ctxt->values[0], ctxt->values[1], ctxt->values[2], ctxt->values[3]);
-}
-
-static void
-_emit_smooth_curve (LsmSvgPathContext *ctxt, gboolean relative)
-{
-	double x, y;
-	double x0, y0;
-
-	cairo_get_current_point (ctxt->cr, &x0, &y0);
-
-	switch (ctxt->last_command) {
-		case 'C':
-			x = 2 * x0 - ctxt->values[2];
-			y = 2 * y0 - ctxt->values[3];
-			break;
-		case 'c':
-			x = 2 * x0 - (ctxt->values[2] + x0 - ctxt->values[4]);
-			y = 2 * y0 - (ctxt->values[3] + y0 - ctxt->values[5]);
-			break;
-		case 'S':
-			x = 2 * x0 - ctxt->values[0];
-			y = 2 * y0 - ctxt->values[1];
-			break;
-		case 's':
-			x = 2 * x0 - (ctxt->values[0] + x0 - ctxt->values[2]);
-			y = 2 * y0 - (ctxt->values[1] + y0 - ctxt->values[3]);
-			break;
-		default: x = x0; y = y0; break;
-	}
-
-	while (lsm_str_parse_double_list (&ctxt->ptr, 4, ctxt->values) == 4) {
-		if (relative) {
-			cairo_get_current_point (ctxt->cr, &x0, &y0);
-			cairo_curve_to (ctxt->cr,
-					x, y,
-					x0 + ctxt->values[0], y0 + ctxt->values[1],
-					x0 + ctxt->values[2], y0 + ctxt->values[3]);
-			x = 2 * (x0 + ctxt->values[2]) - (x0 + ctxt->values[0]);
-			y = 2 * (y0 + ctxt->values[3]) - (y0 + ctxt->values[1]);
-		} else {
-			cairo_curve_to (ctxt->cr, x, y,
-					ctxt->values[0], ctxt->values[1], ctxt->values[2], ctxt->values[3]);
-			x = 2 * ctxt->values[2] - ctxt->values[0];
-			y = 2 * ctxt->values[3] - ctxt->values[1];
-		}
-	}
-}
-
-static void
-_emit_smooth_quadratic_curve (LsmSvgPathContext *ctxt, gboolean relative)
-{
-	double x, y;
-	double x0, y0;
-
-	cairo_get_current_point (ctxt->cr, &x0, &y0);
-
-	switch (ctxt->last_command) {
-		case 'Q':
-			ctxt->last_cp_x = ctxt->values[0];
-			ctxt->last_cp_y = ctxt->values[1];
-			break;
-		case 'q':
-			ctxt->last_cp_x = ctxt->values[0] + x0 - ctxt->values[2];
-			ctxt->last_cp_y = ctxt->values[1] + y0 - ctxt->values[3];
-			break;
-		case 'T':
-		case 't':
-			break;
-		default: ctxt->last_cp_x = x0; ctxt->last_cp_y = y0; break;
-	}
-
-	while (lsm_str_parse_double_list (&ctxt->ptr, 2, ctxt->values) == 2) {
-		x = 2 * x0 - ctxt->last_cp_x;
-		y = 2 * y0 - ctxt->last_cp_y;
-		if (relative) {
-			cairo_get_current_point (ctxt->cr, &x0, &y0);
-			_cairo_quadratic_curve_to (ctxt->cr, x, y, x0 + ctxt->values[0], y0 + ctxt->values[1]);
-		} else {
-			_cairo_quadratic_curve_to (ctxt->cr, x, y, ctxt->values[0], ctxt->values[1]);
-		}
-		ctxt->last_cp_x = x;
-		ctxt->last_cp_y = y;
-		cairo_get_current_point (ctxt->cr, &x0, &y0);
-	}
-}
-
-static void
-_emit_function_6 (LsmSvgPathContext *ctxt,
-		  void (*cairo_func) (cairo_t *, double, double, double ,double, double, double))
-{
-	while (lsm_str_parse_double_list (&ctxt->ptr, 6, ctxt->values) == 6)
-		cairo_func (ctxt->cr, ctxt->values[0], ctxt->values[1], ctxt->values[2],
-			              ctxt->values[3], ctxt->values[4], ctxt->values[5]);
-}
-
-static void
-_emit_function_7 (LsmSvgPathContext *ctxt,
-		  void (*cairo_func) (cairo_t *, double, double, double ,gboolean, gboolean, double, double))
-{
-	while (lsm_str_parse_double_list (&ctxt->ptr, 7, ctxt->values) == 7)
-		cairo_func (ctxt->cr, ctxt->values[0], ctxt->values[1], ctxt->values[2],
-			              ctxt->values[3], ctxt->values[4], ctxt->values[5],
-				      ctxt->values[6]);
-}
-
-static void
-_emit_svg_path (cairo_t *cr, char const *path)
-{
-	LsmSvgPathContext ctxt;
-
-	g_return_if_fail (cr != NULL);
-
-	if (path == NULL)
-		return;
-
-	ctxt.cr = cr;
-	ctxt.ptr = (char *) path;
-	ctxt.last_command = '\0';
-	cairo_get_current_point (cr,
-				 &ctxt.values[0],
-				 &ctxt.values[1]);
-	ctxt.values[2] = ctxt.values[4] = ctxt.values[0];
-	ctxt.values[3] = ctxt.values[5] = ctxt.values[1];
-
-	lsm_str_skip_spaces (&ctxt.ptr);
-
-	while (*ctxt.ptr != '\0') {
-		char command;
-
-		command = *ctxt.ptr;
-		ctxt.ptr++;
-		lsm_str_skip_spaces (&ctxt.ptr);
-
-		if (!cairo_has_current_point (cr)) {
-			cairo_move_to (cr, 0, 0);
-			ctxt.values[2] = ctxt.values[4] = ctxt.values[0] = 0;
-			ctxt.values[3] = ctxt.values[5] = ctxt.values[1] = 0;
-		}
-
-		switch (command) {
-			case 'M': _emit_function_2 (&ctxt, cairo_move_to, cairo_line_to); break;
-			case 'm': _emit_function_2 (&ctxt, cairo_rel_move_to, cairo_rel_line_to); break;
-			case 'L': _emit_function_2 (&ctxt, cairo_line_to, cairo_line_to); break;
-			case 'l': _emit_function_2 (&ctxt, cairo_rel_line_to, cairo_rel_line_to); break;
-			case 'C': _emit_function_6 (&ctxt, cairo_curve_to); break;
-			case 'c': _emit_function_6 (&ctxt, cairo_rel_curve_to); break;
-			case 'S': _emit_smooth_curve (&ctxt, FALSE); break;
-			case 's': _emit_smooth_curve (&ctxt, TRUE); break;
-			case 'V': _emit_function_1 (&ctxt, _cairo_vertical); break;
-			case 'v': _emit_function_1 (&ctxt, _cairo_rel_vertical); break;
-			case 'H': _emit_function_1 (&ctxt, _cairo_horizontal); break;
-			case 'h': _emit_function_1 (&ctxt, _cairo_rel_horizontal); break;
-			case 'Q': _emit_function_4 (&ctxt, _cairo_quadratic_curve_to); break;
-			case 'q': _emit_function_4 (&ctxt, _cairo_rel_quadratic_curve_to); break;
-			case 'T': _emit_smooth_quadratic_curve (&ctxt, FALSE); break;
-			case 't': _emit_smooth_quadratic_curve (&ctxt, TRUE); break;
-			case 'A': _emit_function_7 (&ctxt, _cairo_elliptical_arc); break;
-			case 'a': _emit_function_7 (&ctxt, _cairo_rel_elliptical_arc); break;
-			case 'Z':
-			case 'z': cairo_close_path (cr); break;
-			default: break;
-		}
-
-		ctxt.last_command = command;
-	}
-}
-
 typedef enum {
 	LSM_SVG_VIEW_PAINT_OPERATION_FILL,
 	LSM_SVG_VIEW_PAINT_OPERATION_STROKE
@@ -1074,23 +660,22 @@ paint (LsmSvgView *view, LsmSvgViewPathInfos *path_infos)
 	cairo = view->dom_view.cairo;
 	style = view->style;
 
-#if 0
-	/* This is disabled for now, as care should be taken in push_composition to
-	 * defer group opacity for simple shape to this function. */
-	if (style->opacity != NULL) {
+	if (style->opacity != NULL &&
+	    style->ignore_group_opacity &&
+	    g_strcmp0 (style->filter->value, "none") == 0) {
 		group_opacity = style->opacity->value;
 
-		use_group = style->fill->paint.type != LSM_SVG_PAINT_TYPE_NONE &&
-			style->stroke->paint.type != LSM_SVG_PAINT_TYPE_NONE &&
+		use_group = ((style->fill->paint.type != LSM_SVG_PAINT_TYPE_NONE &&
+			      style->stroke->paint.type != LSM_SVG_PAINT_TYPE_NONE) ||
+			     style->stroke->paint.type == LSM_SVG_PAINT_TYPE_URI ||
+			     style->stroke->paint.type == LSM_SVG_PAINT_TYPE_URI_RGB_COLOR ||
+			     style->fill->paint.type == LSM_SVG_PAINT_TYPE_URI_RGB_COLOR ||
+			     style->fill->paint.type == LSM_SVG_PAINT_TYPE_URI) &&
 			group_opacity < 1.0;
 	} else {
 		use_group = FALSE;
 		group_opacity = 1.0;
 	}
-#else
-	use_group = FALSE;
-	group_opacity = 1.0;
-#endif
 
 	/* Instead of push_group, we should restrict to the current path bounding box */
 	if (use_group)
@@ -1230,13 +815,13 @@ lsm_svg_view_show_rectangle (LsmSvgView *view,
 
 		cairo_move_to (cairo, x + rx, y);
 		cairo_line_to (cairo, x + w - rx, y);
-		_cairo_elliptical_arc (cairo, rx, ry, 0.0, FALSE, TRUE, x + w, y + ry);
+		lsm_cairo_elliptical_arc (cairo, rx, ry, 0.0, FALSE, TRUE, x + w, y + ry);
 		cairo_line_to (cairo, x + w, y + h -ry);
-		_cairo_elliptical_arc (cairo, rx, ry, 0.0, FALSE, TRUE, x + w - rx, y + h);
+		lsm_cairo_elliptical_arc (cairo, rx, ry, 0.0, FALSE, TRUE, x + w - rx, y + h);
 		cairo_line_to (cairo, x + rx, y + h);
-		_cairo_elliptical_arc (cairo, rx, ry, 0.0, FALSE, TRUE, x, y + h - ry);
+		lsm_cairo_elliptical_arc (cairo, rx, ry, 0.0, FALSE, TRUE, x, y + h - ry);
 		cairo_line_to (cairo, x, y + ry);
-		_cairo_elliptical_arc (cairo, rx, ry, 0.0, FALSE, TRUE, x + rx, y);
+		lsm_cairo_elliptical_arc (cairo, rx, ry, 0.0, FALSE, TRUE, x + rx, y);
 		cairo_close_path (cairo);
 	}
 
@@ -1288,34 +873,30 @@ lsm_svg_view_show_path (LsmSvgView *view,
 
 	g_return_if_fail (LSM_IS_SVG_VIEW (view));
 
-	_emit_svg_path (view->dom_view.cairo, d);
+	lsm_cairo_emit_svg_path (view->dom_view.cairo, d);
 
 	process_path (view, &path_infos);
 }
 
 void
-lsm_svg_view_calculate_path_extents (LsmSvgView *view,
-				     const char *path,
-				     double *x1, double *y1,
-				     double *x2, double *y2)
+lsm_svg_view_path_extents (LsmSvgView *view,
+			   const char *path,
+			   LsmExtents *extents)
 {
-	double xx1, yy1, xx2, yy2;
+	double x1, y1, x2, y2;
 
 	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+	g_return_if_fail (extents != NULL);
 
 	cairo_new_path (view->dom_view.cairo);
-	_emit_svg_path (view->dom_view.cairo, path);
-	cairo_path_extents (view->dom_view.cairo, &xx1, &yy1, &xx2, &yy2);
+	lsm_cairo_emit_svg_path (view->dom_view.cairo, path);
+	cairo_path_extents (view->dom_view.cairo, &x1, &y1, &x2, &y2);
 	cairo_new_path (view->dom_view.cairo);
 
-	if (x1 != NULL)
-		*x1 = xx1;
-	if (y1 != NULL)
-		*y1 = yy1;
-	if (x2 != NULL)
-		*x2 = xx2;
-	if (y2 != NULL)
-		*y2 = yy2;
+	extents->x1 = x1;
+	extents->x2 = x2;
+	extents->y1 = y1;
+	extents->y2 = y2;
 }
 
 void
@@ -1379,10 +960,9 @@ lsm_svg_view_show_polygon (LsmSvgView *view, const char *points)
 	_show_points (view, points, TRUE);
 }
 
-void
-lsm_svg_view_show_text (LsmSvgView *view, char const *string, double x, double y)
+static void
+_update_pango_layout (LsmSvgView *view, char const *string, double x, double y, LsmSvgViewPathInfos *path_infos)
 {
-	LsmSvgViewPathInfos path_infos = default_path_infos;
 	const LsmSvgStyle *style;
 	PangoLayout *pango_layout;
 	PangoFontDescription *font_description;
@@ -1393,26 +973,8 @@ lsm_svg_view_show_text (LsmSvgView *view, char const *string, double x, double y
 	int baseline;
 	double x1, y1;
 
-	if (string == NULL || string[0] == '\0')
-		return;
-
-	g_return_if_fail (LSM_IS_SVG_VIEW (view));
-
 	style = view->style;
 
-	lsm_debug_render ("[LsmSvgView::show_text] Show '%s' at %g,%g (%g px)", string, x, y, style->font_size_px);
-
-	if (view->is_pango_layout_in_use) {
-		PangoContext *pango_context;
-
-		pango_context = pango_layout_get_context (view->pango_layout);
-		view->pango_layout_stack = g_slist_prepend (view->pango_layout_stack, view->pango_layout);
-		view->pango_layout = pango_layout_new (pango_context);
-
-		lsm_debug_render ("[LsmSvgView::show_text] Create a new pango layout");
-	} else
-		view->is_pango_layout_in_use = TRUE;
-
 	pango_layout = view->pango_layout;
 	font_description = view->dom_view.font_description;
 
@@ -1489,30 +1051,42 @@ lsm_svg_view_show_text (LsmSvgView *view, char const *string, double x, double y
 			break;
 	}
 
-	path_infos.is_text_path = TRUE;
-	path_infos.is_extents_defined = TRUE;
-	path_infos.extents.x1 = x1;
-	path_infos.extents.y1 = y1;
-	path_infos.extents.x2 = x1 + pango_units_to_double (rectangle.width);
-	path_infos.extents.y2 = y1 + pango_units_to_double (rectangle.height);
-	path_infos.pango_layout = pango_layout;
+	path_infos->is_text_path = TRUE;
+	path_infos->is_extents_defined = TRUE;
+	path_infos->extents.x1 = x1;
+	path_infos->extents.y1 = y1;
+	path_infos->extents.x2 = x1 + pango_units_to_double (rectangle.width);
+	path_infos->extents.y2 = y1 + pango_units_to_double (rectangle.height);
+	path_infos->pango_layout = pango_layout;
+}
 
-	if (style->writing_mode->value == LSM_SVG_WRITING_MODE_TB ||
-	    style->writing_mode->value == LSM_SVG_WRITING_MODE_TB_RL) {
+static gboolean
+_lock_pango_layout (LsmSvgView *view)
+{
+	/* A text may be painted with a text based pattern. In this case,
+	 * we take care to create a new pango layout if the current one is in use. */
+	if (view->is_pango_layout_in_use) {
+		PangoContext *pango_context;
 
-		cairo_save (view->dom_view.cairo);
-		cairo_rotate (view->dom_view.cairo, M_PI / 2.0);
-		cairo_move_to (view->dom_view.cairo, x1, y1);
+		pango_context = pango_layout_get_context (view->pango_layout);
+		view->pango_layout_stack = g_slist_prepend (view->pango_layout_stack, view->pango_layout);
+		view->pango_layout = pango_layout_new (pango_context);
 
-		process_path (view, &path_infos);
+		lsm_debug_render ("[LsmSvgView::show_text] Create a new pango layout");
 
-		cairo_restore (view->dom_view.cairo);
-	} else {
-		cairo_move_to (view->dom_view.cairo, x1, y1);
-		process_path (view, &path_infos);
+		return TRUE;
 	}
 
-	if (pango_layout != view->pango_layout) {
+	view->is_pango_layout_in_use = TRUE;
+
+	return FALSE;
+}
+
+static void
+_unlock_pango_layout (LsmSvgView *view, gboolean need_pop)
+{
+
+	if (need_pop) {
 		lsm_debug_render ("[LsmSvgView::show_text] Free the child pango layout");
 
 		if (view->pango_layout != NULL) {
@@ -1528,6 +1102,70 @@ lsm_svg_view_show_text (LsmSvgView *view, char const *string, double x, double y
 	view->is_pango_layout_in_use = FALSE;
 }
 
+void
+lsm_svg_view_show_text (LsmSvgView *view, char const *string, double x, double y)
+{
+	LsmSvgViewPathInfos path_infos = default_path_infos;
+	const LsmSvgStyle *style;
+	gboolean need_pop;
+
+	if (string == NULL || string[0] == '\0')
+		return;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+
+	style = view->style;
+
+	lsm_debug_render ("[LsmSvgView::show_text] Show '%s' at %g,%g (%g px)", string, x, y, style->font_size_px);
+
+	need_pop = _lock_pango_layout (view);
+
+	_update_pango_layout (view, string, x, y, &path_infos);
+
+	if (style->writing_mode->value == LSM_SVG_WRITING_MODE_TB ||
+	    style->writing_mode->value == LSM_SVG_WRITING_MODE_TB_RL) {
+
+		cairo_save (view->dom_view.cairo);
+		cairo_rotate (view->dom_view.cairo, M_PI / 2.0);
+		cairo_move_to (view->dom_view.cairo, path_infos.extents.x1, path_infos.extents.y1);
+
+		process_path (view, &path_infos);
+
+		cairo_restore (view->dom_view.cairo);
+	} else {
+		cairo_move_to (view->dom_view.cairo, path_infos.extents.x1, path_infos.extents.y1);
+		process_path (view, &path_infos);
+	}
+
+	_unlock_pango_layout (view, need_pop);
+}
+
+void
+lsm_svg_view_text_extents (LsmSvgView *view, char const *string, double x, double y, LsmExtents *extents)
+{
+	LsmSvgViewPathInfos path_infos = default_path_infos;
+	gboolean need_pop;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+	g_return_if_fail (extents != NULL);
+
+	if (string == NULL ||
+	    string[0] == '\0') {
+		extents->x1 = 0;
+		extents->y1 = 0;
+		extents->y1 = 0;
+		extents->y2 = 0;
+	}
+
+	need_pop = _lock_pango_layout (view);
+
+	_update_pango_layout (view, string, x, y, &path_infos);
+
+	_unlock_pango_layout (view, need_pop);
+
+	*extents = path_infos.extents;
+}
+
 void
 lsm_svg_view_show_pixbuf (LsmSvgView *view, GdkPixbuf *pixbuf)
 {
@@ -1963,7 +1601,7 @@ void
 lsm_svg_view_pop_filter (LsmSvgView *view)
 {
 	LsmSvgElement *filter_element;
-	LsmFilterSurface *filter_surface;
+	LsmSvgFilterSurface *filter_surface;
 	cairo_surface_t *surface;
 	GSList *iter;
 
@@ -1975,11 +1613,18 @@ lsm_svg_view_pop_filter (LsmSvgView *view)
 	if (LSM_IS_SVG_FILTER_ELEMENT (filter_element) &&
 	    view->pattern_data->pattern != NULL) {
 		cairo_matrix_t matrix;
+		LsmBox subregion;
 
 		view->filter_surfaces = NULL;
 
 		cairo_pattern_get_surface (view->pattern_data->pattern, &surface);
-		filter_surface = lsm_filter_surface_new_with_content ("SourceGraphic", 0, 0, surface);
+
+		subregion.x = 0;
+		subregion.y = 0;
+		subregion.width = cairo_image_surface_get_width (surface);
+		subregion.height = cairo_image_surface_get_height (surface);
+
+		filter_surface = lsm_svg_filter_surface_new_with_content ("SourceGraphic", surface, &subregion);
 		cairo_pattern_get_matrix (view->pattern_data->pattern, &matrix);
 
 		view->filter_surfaces = g_slist_prepend (view->filter_surfaces, filter_surface);
@@ -1992,12 +1637,12 @@ lsm_svg_view_pop_filter (LsmSvgView *view)
 			static int count = 0;
 
 			for (iter = view->filter_surfaces; iter != NULL; iter = iter->next) {
-				LsmFilterSurface *surface = iter->data;
+				LsmSvgFilterSurface *surface = iter->data;
 
 				filename = g_strdup_printf ("filter-%04d-%s-%s.png", count++,
 							    view->style->filter->value,
-							    lsm_filter_surface_get_name (surface));
-				cairo_surface_write_to_png (lsm_filter_surface_get_cairo_surface (surface), filename);
+							    lsm_svg_filter_surface_get_name (surface));
+				cairo_surface_write_to_png (lsm_svg_filter_surface_get_cairo_surface (surface), filename);
 				g_free (filename);
 			}
 		}
@@ -2006,16 +1651,17 @@ lsm_svg_view_pop_filter (LsmSvgView *view)
 			cairo_pattern_t *pattern;
 			cairo_surface_t *surface;
 
-			surface = lsm_filter_surface_get_cairo_surface (view->filter_surfaces->data);
+			surface = lsm_svg_filter_surface_get_cairo_surface (view->filter_surfaces->data);
 			pattern = cairo_pattern_create_for_surface (surface);
 			cairo_pattern_set_extend (pattern, CAIRO_EXTEND_NONE);
 			cairo_pattern_set_matrix (pattern, &matrix);
 			cairo_set_source (view->pattern_data->old_cairo, pattern);
+			cairo_pattern_destroy (pattern);
 			cairo_paint_with_alpha (view->pattern_data->old_cairo, view->style->opacity->value);
 		}
 
 		for (iter = view->filter_surfaces; iter != NULL; iter = iter->next)
-			lsm_filter_surface_unref (iter->data);
+			lsm_svg_filter_surface_unref (iter->data);
 		g_slist_free (view->filter_surfaces);
 		view->filter_surfaces = NULL;
 	}
@@ -2023,68 +1669,138 @@ lsm_svg_view_pop_filter (LsmSvgView *view)
 	_end_pattern (view);
 }
 
-static LsmFilterSurface *
+static LsmSvgFilterSurface *
 _get_filter_surface (LsmSvgView *view, const char *input)
 {
 	GSList *iter;
-	LsmFilterSurface *source_surface = NULL;
+	LsmSvgFilterSurface *source_surface = NULL;
 
 	if (input == NULL)
 		return view->filter_surfaces->data;
 
 	for (iter = view->filter_surfaces; iter != NULL; iter = iter->next) {
-		LsmFilterSurface *surface = iter->data;
+		LsmSvgFilterSurface *surface = iter->data;
 
-		if (g_strcmp0 (input, lsm_filter_surface_get_name (surface)) == 0)
+		if (g_strcmp0 (input, lsm_svg_filter_surface_get_name (surface)) == 0)
 			return surface;
 
 		source_surface = surface;
 	}
 
 	if (g_strcmp0 (input, "SourceAlpha") == 0 && source_surface != NULL) {
-		LsmFilterSurface *surface;
+		LsmSvgFilterSurface *surface;
 
-		surface = lsm_filter_surface_new_similar ("SourceAlpha", source_surface);
-		lsm_filter_surface_alpha (source_surface, surface);
+		surface = lsm_svg_filter_surface_new_similar ("SourceAlpha", source_surface, NULL);
+		lsm_svg_filter_surface_alpha (source_surface, surface);
 		view->filter_surfaces = g_slist_prepend (view->filter_surfaces, surface);	
 
 		return surface;
 	} else if (g_strcmp0 (input, "BackgroundImage") == 0) {
-		/* TODO */
-		LsmFilterSurface *surface;
+		LsmSvgFilterSurface *surface;
+		LsmSvgViewBackground *background;
+		gboolean background_processing = FALSE;
+		cairo_matrix_t matrix;
+		cairo_matrix_t pattern_matrix;
+		cairo_t *cairo;
+		GList *iter;
 
-		surface = lsm_filter_surface_new_similar ("BackgroundImage", source_surface);
+		for (iter = view->background_stack; iter != NULL; iter = iter->next) {
+			background = iter->data;
+
+			if (background->enable_background) {
+				background_processing = TRUE;
+				break;
+			}
+		}
+
+		if (!background_processing) {
+			lsm_debug_render ("[LsmSvgView::_get_filter_surface] Background processing not enabled");
+			return NULL;
+		}
+
+		surface = lsm_svg_filter_surface_new_similar ("BackgroundImage", source_surface, NULL);
 		view->filter_surfaces = g_slist_prepend (view->filter_surfaces, surface);	
+
+		cairo_get_matrix (view->pattern_data->old_cairo, &matrix);
+		cairo_pattern_get_matrix (view->pattern_data->pattern, &pattern_matrix);
+		
+		cairo_matrix_invert (&matrix);
+		cairo_matrix_multiply (&matrix, &matrix, &pattern_matrix);
+
+		lsm_debug_render ("[LsmSvgView::_get_filter_surface] Background image matrix %g, %g, %g, %g, %g, %g",
+				  matrix.xx, matrix.xy, matrix.yx, matrix.yy,
+				  matrix.x0, matrix.y0);
+
+		cairo = cairo_create (lsm_svg_filter_surface_get_cairo_surface (surface));
+		cairo_set_matrix (cairo, &matrix);
+
+		for (; iter != NULL; iter = iter->prev) {
+			background = iter->data;
+
+			cairo_set_source_surface (cairo, background->surface, 0, 0);
+			cairo_paint_with_alpha (cairo, background->group_opacity);
+		}
+
+		cairo_destroy (cairo);
+		
+		return surface;
 	} else if (g_strcmp0 (input, "BackgroundAlpha") == 0) {
-		/* TODO */
-		LsmFilterSurface *surface;
+		LsmSvgFilterSurface *surface;
+		LsmSvgFilterSurface *background_surface;
 
-		surface = lsm_filter_surface_new_similar ("BackgroundAlpha", source_surface);
+		if (view->background_stack == NULL)
+			return NULL;
+
+		background_surface = _get_filter_surface (view, "BackgroundImage");
+
+		surface = lsm_svg_filter_surface_new_similar ("BackgroundAlpha", background_surface, NULL);
+		lsm_svg_filter_surface_alpha (background_surface, surface);
 		view->filter_surfaces = g_slist_prepend (view->filter_surfaces, surface);	
+
+		return surface;
 	}
 
 	return NULL;
 }
 
-static LsmFilterSurface *
-_create_filter_surface (LsmSvgView *view, const char *output, LsmFilterSurface *input_surface)
+static LsmSvgFilterSurface *
+_create_filter_surface (LsmSvgView *view, const char *output, LsmSvgFilterSurface *input_surface, const LsmBox *subregion)
 {
-	LsmFilterSurface *surface;
+	LsmSvgFilterSurface *surface;
 
-	surface = lsm_filter_surface_new_similar (output, input_surface);
+	surface = lsm_svg_filter_surface_new_similar (output, input_surface, subregion);
 
 	view->filter_surfaces = g_slist_prepend (view->filter_surfaces, surface); 
 
 	return surface;
 }
 
+LsmBox
+lsm_svg_view_get_filter_surface_extents (LsmSvgView *view, const char *name)
+{
+	static LsmBox null_extents = {.x = 0.0, .y = 0.0, .width = 0.0, .height = 0.0};
+	LsmSvgFilterSurface *surface;
+	LsmBox extents;
+
+	g_return_val_if_fail (LSM_IS_SVG_VIEW (view), null_extents);
+
+	surface = _get_filter_surface (view, name);
+	if (surface == NULL)
+		return null_extents;
+
+	lsm_cairo_box_device_to_user (view->dom_view.cairo, &extents, lsm_svg_filter_surface_get_subregion (surface));
+
+	return extents;
+}
+
 void
 lsm_svg_view_apply_blend (LsmSvgView *view, const char *input_1, const char*input_2, const char *output,
-			  LsmSvgBlendingMode mode)
+			  const LsmBox *subregion, LsmSvgBlendingMode mode)
 {
-	LsmFilterSurface *output_surface;
-	LsmFilterSurface *input_1_surface;
-	LsmFilterSurface *input_2_surface;
+	LsmSvgFilterSurface *output_surface;
+	LsmSvgFilterSurface *input_1_surface;
+	LsmSvgFilterSurface *input_2_surface;
+	LsmBox subregion_px;
 
 	g_return_if_fail (LSM_IS_SVG_VIEW (view));
 
@@ -2092,52 +1808,50 @@ lsm_svg_view_apply_blend (LsmSvgView *view, const char *input_1, const char*inpu
 	input_2_surface = _get_filter_surface (view, input_2);
 
 	if (input_1_surface == NULL || input_2_surface == NULL) {
-		lsm_debug_render ("[SvgView::apply_blend] Inputs '%s' or '%s' not found", input_1, input_2);
+		lsm_warning_render ("[SvgView::apply_blend] Inputs '%s' or '%s' not found", input_1, input_2);
 		return;
 	}
 
-	output_surface = _create_filter_surface (view, output, input_1_surface);
+	lsm_cairo_box_user_to_device (view->dom_view.cairo, &subregion_px, subregion);
+	output_surface = _create_filter_surface (view, output, input_1_surface, &subregion_px);
 
 	lsm_log_render ("[SvgView::blend] mode = %s", lsm_svg_blending_mode_to_string (mode));
 
-	lsm_filter_surface_blend (input_1_surface, input_2_surface, output_surface, mode);
+	lsm_svg_filter_surface_blend (input_1_surface, input_2_surface, output_surface, mode);
 }
 
 void
-lsm_svg_view_apply_flood (LsmSvgView *view, const char *output,
-			  double x, double y, double w, double h)
+lsm_svg_view_apply_flood (LsmSvgView *view, const char *output, const LsmBox *subregion)
 {
-	LsmFilterSurface *output_surface;
-	LsmFilterSurface *input_surface;
-	guint8 red, green, blue;
-	guint32 color;
-	double opacity;
+	LsmSvgFilterSurface *output_surface;
+	LsmSvgFilterSurface *input_surface;
+	LsmBox subregion_px;
 
 	g_return_if_fail (LSM_IS_SVG_VIEW (view));
 
 	input_surface = _get_filter_surface (view, NULL);
 
-	output_surface = _create_filter_surface (view, output, input_surface);
+	lsm_cairo_box_user_to_device (view->dom_view.cairo, &subregion_px, subregion);
+	output_surface = _create_filter_surface (view, output, input_surface, &subregion_px);
 
-	red = (double) (0.5 + view->style->flood_color->value.red * 255.0);
-	green = (double) (0.5 + view->style->flood_color->value.green * 255.0);
-	blue = (double) (0.5 + view->style->flood_color->value.blue * 255.0);
-	color = red << 16 | green << 8 | blue << 0;
-	opacity = view->style->flood_opacity->value;
+	lsm_log_render ("[SvgView::apply_flood] subregion %gx%g px at %g,%g px",
+		        subregion_px.width, subregion_px.height,
+		        subregion_px.x, subregion_px.y);
 
-	lsm_log_render ("[SvgView::apply_flood] color = 0x%06x - opacity = %g",
-			color, opacity);
-
-	lsm_filter_surface_flood (output_surface, color, opacity);
+	lsm_svg_filter_surface_flood (output_surface,
+				  view->style->flood_color->value.red,
+				  view->style->flood_color->value.green,
+				  view->style->flood_color->value.blue,
+				  view->style->flood_opacity->value);
 }
 
 void
 lsm_svg_view_apply_gaussian_blur (LsmSvgView *view, const char *input, const char *output,
-				  double x, double y, double w, double h,
-				  double std_x, double std_y)
+				  const LsmBox *subregion, double std_x, double std_y)
 {
-	LsmFilterSurface *input_surface;
-	LsmFilterSurface *output_surface;
+	LsmSvgFilterSurface *input_surface;
+	LsmSvgFilterSurface *output_surface;
+	LsmBox subregion_px;
 
 	g_return_if_fail (LSM_IS_SVG_VIEW (view));
 
@@ -2148,26 +1862,29 @@ lsm_svg_view_apply_gaussian_blur (LsmSvgView *view, const char *input, const cha
 		return;
 	}
 
-	output_surface = _create_filter_surface (view, output, input_surface);
+	lsm_cairo_box_user_to_device (view->dom_view.cairo, &subregion_px, subregion);
+	output_surface = _create_filter_surface (view, output, input_surface, &subregion_px);
 
-	lsm_log_render ("[SvgView::apply_gaussian_blur] %s -> %s (x:%g,y:%g,w:%g,h:%g) (%g,%g)",
+	lsm_log_render ("[SvgView::apply_gaussian_blur] %s -> %s (%g,%g)",
 			input != NULL ? input : "previous",
 			output != NULL ? output : "next",
-			x, y, w, h, std_x, std_y); 
+			std_x, std_y); 
 
 	cairo_user_to_device_distance (view->dom_view.cairo, &std_x, &std_y);
 
 	lsm_log_render ("[SvgView::apply_gaussian_blur] %g px,%g px",
 			std_x, std_y);
 
-	lsm_filter_surface_fast_blur (input_surface, output_surface, std_x, std_y);
+	lsm_svg_filter_surface_fast_blur (input_surface, output_surface, std_x, std_y);
 }
 
 void
-lsm_svg_view_apply_offset (LsmSvgView *view, const char *input, const char *output, double dx, double dy)
+lsm_svg_view_apply_offset (LsmSvgView *view, const char *input, const char *output,
+			   const LsmBox *subregion, double dx, double dy)
 {
-	LsmFilterSurface *input_surface;
-	LsmFilterSurface *output_surface;
+	LsmSvgFilterSurface *input_surface;
+	LsmSvgFilterSurface *output_surface;
+	LsmBox subregion_px;
 
 	g_return_if_fail (LSM_IS_SVG_VIEW (view));
 
@@ -2178,7 +1895,8 @@ lsm_svg_view_apply_offset (LsmSvgView *view, const char *input, const char *outp
 		return;
 	}
 
-	output_surface = _create_filter_surface (view, output, input_surface);
+	lsm_cairo_box_user_to_device (view->dom_view.cairo, &subregion_px, subregion);
+	output_surface = _create_filter_surface (view, output, input_surface, &subregion_px);
 
 	lsm_log_render ("[SvgView::apply_offset] %s -> %s (dx:%g,dy:%g)", input, output, dx, dy); 
 
@@ -2186,14 +1904,15 @@ lsm_svg_view_apply_offset (LsmSvgView *view, const char *input, const char *outp
 
 	lsm_log_render ("[SvgView::apply_offset] %g px,%g px", dx, dy);
 
-	lsm_filter_surface_offset (input_surface, output_surface, dx, dy);
+	lsm_svg_filter_surface_offset (input_surface, output_surface, dx, dy);
 }
 
 void
-lsm_svg_view_apply_merge (LsmSvgView *view, const char *input, const char *output)
+lsm_svg_view_apply_merge (LsmSvgView *view, const char *input, const char *output, const LsmBox *subregion)
 {
-	LsmFilterSurface *input_surface;
-	LsmFilterSurface *output_surface;
+	LsmSvgFilterSurface *input_surface;
+	LsmSvgFilterSurface *output_surface;
+	LsmBox subregion_px;
 
 	g_return_if_fail (LSM_IS_SVG_VIEW (view));
 
@@ -2205,11 +1924,32 @@ lsm_svg_view_apply_merge (LsmSvgView *view, const char *input, const char *outpu
 	}
 
 	output_surface = _get_filter_surface (view, output);
+	lsm_cairo_box_user_to_device (view->dom_view.cairo, &subregion_px, subregion);
 	if (output_surface == NULL)
-		output_surface = _create_filter_surface (view, output, input_surface);
+		output_surface = _create_filter_surface (view, output, input_surface, &subregion_px);
 
 	if (output_surface != NULL)
-		lsm_filter_surface_merge (input_surface, output_surface);
+		lsm_svg_filter_surface_merge (input_surface, output_surface);
+}
+
+void
+lsm_svg_view_apply_tile (LsmSvgView *view, const char *input, const char *output, const LsmBox *subregion)
+{
+	LsmSvgFilterSurface *input_surface;
+	LsmSvgFilterSurface *output_surface;
+	LsmBox subregion_px;
+
+	input_surface = _get_filter_surface (view, input);
+
+	if (input_surface == NULL) {
+		lsm_debug_render ("[SvgView::apply_offset] Input '%s' not found", input);
+		return;
+	}
+
+	lsm_cairo_box_user_to_device (view->dom_view.cairo, &subregion_px, subregion);
+	output_surface = _create_filter_surface (view, output, input_surface, &subregion_px);
+
+	lsm_svg_filter_surface_tile (input_surface, output_surface);
 }
 
 void
@@ -2304,9 +2044,24 @@ lsm_svg_view_push_composition (LsmSvgView *view, LsmSvgStyle *style)
 	do_mask = (g_strcmp0 (style->mask->value, "none") != 0);
 	do_filter = (g_strcmp0 (style->filter->value, "none") != 0);
 
-	if (view->style->opacity->value < 1.0 && !do_filter)
+	if ((view->style->opacity->value < 1.0 ||
+	     view->style->enable_background->value == LSM_SVG_ENABLE_BACKGROUND_NEW) &&
+	    !do_filter &&
+	    !view->is_clipping &&
+	    !view->style->ignore_group_opacity) {
+		LsmSvgViewBackground *background;
+
+		lsm_debug_render ("[LsmSvgView::push_composition] Push group");
 		cairo_push_group (view->dom_view.cairo);
 
+		background = g_slice_new (LsmSvgViewBackground);
+		background->surface = cairo_get_group_target (view->dom_view.cairo);
+		background->group_opacity = view->style->opacity->value;
+		background->enable_background = view->style->enable_background->value == LSM_SVG_ENABLE_BACKGROUND_NEW;
+
+		view->background_stack = g_list_prepend (view->background_stack, background);
+	}
+
 	if (do_clip) {
 		lsm_debug_render ("[LsmSvgView::push_style] Start clip '%s'", style->clip_path->value);
 		lsm_svg_view_push_clip (view);
@@ -2364,9 +2119,17 @@ void lsm_svg_view_pop_composition (LsmSvgView *view)
 	if (do_clip)
 		lsm_svg_view_pop_clip (view);
 
-	if (view->style->opacity->value < 1.0 && !do_filter) {
+	if ((view->style->opacity->value < 1.0 ||
+	     view->style->enable_background->value == LSM_SVG_ENABLE_BACKGROUND_NEW) &&
+	    !do_filter &&
+	    !view->is_clipping &&
+	    !view->style->ignore_group_opacity) {
+		g_slice_free (LsmSvgViewBackground, view->background_stack->data);
+		view->background_stack = g_list_delete_link (view->background_stack, view->background_stack);
+
 		cairo_pop_group_to_source (view->dom_view.cairo);
 		cairo_paint_with_alpha (view->dom_view.cairo, view->style->opacity->value);
+		lsm_debug_render ("[LsmSvgView::pop_composition] Pop group");
 	}
 
 	lsm_svg_view_pop_style (view);
@@ -2444,6 +2207,7 @@ lsm_svg_view_render (LsmDomView *view)
 	svg_view->viewbox_stack = NULL;
 	svg_view->matrix_stack = NULL;
 	svg_view->pango_layout_stack = NULL;
+	svg_view->background_stack = NULL;
 
 	svg_view->is_clipping = FALSE;
 	svg_view->is_pango_layout_in_use = FALSE;
@@ -2485,6 +2249,11 @@ lsm_svg_view_render (LsmDomView *view)
 		g_slist_free (svg_view->style_stack);
 		svg_view->style_stack = NULL;
 	}
+	if (svg_view->background_stack != NULL) {
+		g_warning ("[LsmSvgView::render] Dangling background in stack");
+		g_list_free (svg_view->background_stack);
+		svg_view->background_stack = NULL;
+	}
 }
 
 static void
