@@ -60,6 +60,7 @@ struct _LsmSvgView {
 	GSList *viewbox_stack;
 	GSList *matrix_stack;
 	GSList *pango_layout_stack;
+	GList *background_stack;
 
 	gboolean is_pango_layout_in_use;
 
@@ -95,9 +96,7 @@ const LsmBox *	lsm_svg_view_get_pattern_extents	(LsmSvgView *view);
 const LsmBox * 	lsm_svg_view_get_object_extents 	(LsmSvgView *view);
 const LsmBox *	lsm_svg_view_get_clip_extents		(LsmSvgView *view);
 
-void		lsm_svg_view_calculate_path_extents	(LsmSvgView *view, const char *path,
-							 double *x1, double *y1,
-							 double *x2, double *y2);
+void		lsm_svg_view_path_extents		(LsmSvgView *view, const char *path, LsmExtents *extents);
 
 void 		lsm_svg_view_create_radial_gradient 	(LsmSvgView *view, double cx, double cy,
 							                   double r, double fx, double fy);
@@ -125,6 +124,7 @@ void 		lsm_svg_view_show_line 		(LsmSvgView *view, double x1, double y1, double
 void 		lsm_svg_view_show_polyline	(LsmSvgView *view, const char *points);
 void 		lsm_svg_view_show_polygon	(LsmSvgView *view, const char *points);
 void 		lsm_svg_view_show_text 		(LsmSvgView *view, char const *text, double x, double y);
+void 		lsm_svg_view_text_extents 	(LsmSvgView *view, char const *string, double x, double y, LsmExtents *extents);
 void		lsm_svg_view_show_pixbuf	(LsmSvgView *view, GdkPixbuf *pixbuf);
 
 void 		lsm_svg_view_push_viewbox 		(LsmSvgView *view, const LsmBox *viewbox);
@@ -146,20 +146,21 @@ void		lsm_svg_view_pop_element		(LsmSvgView *view);
 
 void		lsm_svg_view_push_style			(LsmSvgView *view, LsmSvgStyle *style);
 void		lsm_svg_view_pop_style			(LsmSvgView *view);
+LsmSvgStyle *	lsm_svg_view_get_current_style		(LsmSvgView *view);
+
 void		lsm_svg_view_push_composition		(LsmSvgView *view, LsmSvgStyle *style);
 void		lsm_svg_view_pop_composition		(LsmSvgView *view);
 
-LsmSvgStyle *	lsm_svg_view_get_current_style	(LsmSvgView *view);
-
+LsmBox 		lsm_svg_view_get_filter_surface_extents (LsmSvgView *view, const char *name);
 void 		lsm_svg_view_apply_blend 		(LsmSvgView *view, const char *input_1, const char*input_2, const char *output,
-							 LsmSvgBlendingMode mode);
-void 		lsm_svg_view_apply_flood 		(LsmSvgView *view, const char *output,
-							 double x, double y, double w, double h);
-void		lsm_svg_view_apply_gaussian_blur 	(LsmSvgView *view, const char *input, const char *output,
-							 double x, double y, double w, double h,
+							 const LsmBox *subregion, LsmSvgBlendingMode mode);
+void 		lsm_svg_view_apply_flood 		(LsmSvgView *view, const char *output, const LsmBox *subregion);
+void		lsm_svg_view_apply_gaussian_blur 	(LsmSvgView *view, const char *input, const char *output, const LsmBox *subregion,
 							 double std_x, double std_y);
-void 		lsm_svg_view_apply_offset 		(LsmSvgView *view, const char *input, const char *output, double dx, double dy);
-void 		lsm_svg_view_apply_merge 		(LsmSvgView *view, const char *input, const char *output);
+void 		lsm_svg_view_apply_offset 		(LsmSvgView *view, const char *input, const char *output, const LsmBox *subregion,
+							 double dx, double dy);
+void 		lsm_svg_view_apply_merge 		(LsmSvgView *view, const char *input, const char *output, const LsmBox *subregion);
+void 		lsm_svg_view_apply_tile 		(LsmSvgView *view, const char *input, const char *output, const LsmBox *subregion);
 
 G_END_DECLS
 
